#include <GL/gl.h>	// Header File For The OpenGL32 Library
#include <GL/glu.h>	// Header File For The GLu32 Library
#include <SDL.h>

#include <math.h>
#include "snf.h"
#include <iostream>
#include <vector>
#include <list>
#include <map>
#include <set>
#include <utility>

#include "vector.h"	// La NML de Arnau

#define PI 3.141592

/*
int unfla1 = false;
int unfla2 = false;
int unfla3 = false;
int unfla4 = false;
*/

typedef nml::vector<3> punto;

struct cara
{
	cara(void):a(0),b(0),c(0){}
	cara(punto *aa,punto *bb,punto *cc):a(aa),b(bb),c(cc){}
	
	punto *a;
	punto *b;
	punto *c;
};

struct figura
{
	std::vector<punto *>              first;
	std::list<cara>                   second;
	std::vector<std::list<punto *> *> third;
	std::vector<punto *>              fourth;
};

void InitGL(int Width, int Height)	        // We call this right after our OpenGL window is created.
{
  glViewport(0, 0, Width, Height);
  glClearColor(0.0f, 0.0f, 0.0f, 0.0f);		// This Will Clear The Background Color To Black
  glClearDepth(1.0f);				// Enables Clearing Of The Depth Buffer
  glDepthFunc(GL_LESS);				// The Type Of Depth Test To Do
  glEnable(GL_DEPTH_TEST);			// Enables Depth Testing
  glShadeModel(GL_SMOOTH);			// Enables Smooth Color Shading

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();				// Reset The Projection Matrix

  gluPerspective(45.0f,(GLfloat)Width/(GLfloat)Height,0.1f,100.0f);	// Calculate The Aspect Ratio Of The Window
  glMatrixMode(GL_MODELVIEW);
}

void unfla(figura *bola)
{
	typedef std::list<cara> caralist;
	typedef std::list<punto *> enlaces;
	typedef std::pair<punto *,enlaces *> puntoen;
	
	typedef std::map
	<
		std::pair
		<
			punto *,
			punto *
		>,
		puntoen
> mapapunto;
	
	caralist back;
	
	mapapunto done;
	mapapunto::iterator j;
	for(caralist::iterator i=bola->second.begin();i!=bola->second.end();++i)
	{
		punto   *ab ,*bc ,*ca ;
		enlaces *eab,*ebc,*eca;
		
		j=done.find(mapapunto::key_type(i->a,i->b));
		if(j==done.end())
		{
			ab =new punto((*(i->a)+*(i->b))/2.0f);
			eab=new enlaces;
			eab->push_back(i->a);
			eab->push_back(i->b);
			bola->first.push_back(ab);
			bola->third.push_back(eab);
			bola->fourth.push_back(new punto(0.0f,0.0f,0.0f));
			
			done.insert(mapapunto::value_type(mapapunto::key_type(i->a,i->b),puntoen(ab,eab)));
		}
		else
		{
			ab =j->second.first;
			eab=j->second.second;
		}
		
		j=done.find(mapapunto::key_type(i->b,i->c));
		if(j==done.end())
		{
			bc =new punto((*(i->b)+*(i->c))/2.0f);
			ebc=new enlaces;
			ebc->push_back(i->b);
			ebc->push_back(i->c);
			bola->first.push_back(bc);
			bola->third.push_back(ebc);
			bola->fourth.push_back(new punto(0.0f,0.0f,0.0f));
			
			done.insert(mapapunto::value_type(mapapunto::key_type(i->b,i->c),puntoen(bc,ebc)));
		}
		else
		{
			bc =j->second.first;
			ebc=j->second.second;
		}
		
		j=done.find(mapapunto::key_type(i->c,i->a));
		if(j==done.end())
		{
			ca =new punto((*(i->c)+*(i->a))/2.0f);
			eca=new enlaces;
			eca->push_back(i->c);
			eca->push_back(i->a);
			bola->first.push_back(ca);
			bola->third.push_back(eca);
			bola->fourth.push_back(new punto(0.0f,0.0f,0.0f));
			
			done.insert(mapapunto::value_type(mapapunto::key_type(i->c,i->a),puntoen(ca,eca)));
		}
		else
		{
			ca =j->second.first;
			eca=j->second.second;
		}

		eab->push_back(bc);
		eab->push_back(ca);
		ebc->push_back(ab);
		ebc->push_back(ca);
		eca->push_back(ab);
		eca->push_back(bc);
		back.push_back(cara(i->a,ab,ca));
		back.push_back(cara(i->b,bc,ab));
		back.push_back(cara(i->c,ca,bc));
		i->a=ab;
		i->b=bc;
		i->c=ca;
	}
	bola->second.splice(bola->second.begin(),back);
}

figura * hacer_bola(void)
{
	figura *nueva=new figura;

	//Empezamos con un octaedro:
	
	nueva->first.push_back(new punto( 2.0f, 0.0f, 0.0f));
	nueva->first.push_back(new punto(-2.0f, 0.0f, 0.0f));
	nueva->first.push_back(new punto( 0.0f, 0.0f, 2.0f));
	nueva->first.push_back(new punto( 0.0f, 0.0f,-2.0f));
	nueva->first.push_back(new punto( 0.0f, 2.0f, 0.0f));
	nueva->first.push_back(new punto( 0.0f,-2.0f, 0.0f));

	nueva->second.push_back(cara(nueva->first[4],nueva->first[0],nueva->first[2]));
	nueva->second.push_back(cara(nueva->first[4],nueva->first[2],nueva->first[1]));
	nueva->second.push_back(cara(nueva->first[4],nueva->first[1],nueva->first[3]));
	nueva->second.push_back(cara(nueva->first[4],nueva->first[3],nueva->first[0]));
	nueva->second.push_back(cara(nueva->first[5],nueva->first[0],nueva->first[2]));
	nueva->second.push_back(cara(nueva->first[5],nueva->first[2],nueva->first[1]));
	nueva->second.push_back(cara(nueva->first[5],nueva->first[1],nueva->first[3]));
	nueva->second.push_back(cara(nueva->first[5],nueva->first[3],nueva->first[0]));

	std::list<punto *> *dummy;
	
	dummy=new std::list<punto *>;
	dummy->push_back(nueva->first[2]);
	dummy->push_back(nueva->first[3]);
	dummy->push_back(nueva->first[4]);
	dummy->push_back(nueva->first[5]);
	nueva->third.push_back(dummy);
	nueva->fourth.push_back(new punto(0.0f,0.0f,0.0f));

	dummy=new std::list<punto *>;
	dummy->push_back(nueva->first[2]);
	dummy->push_back(nueva->first[3]);
	dummy->push_back(nueva->first[4]);
	dummy->push_back(nueva->first[5]);
	nueva->third.push_back(dummy);
	nueva->fourth.push_back(new punto(0.0f,0.0f,0.0f));

	dummy=new std::list<punto *>;
	dummy->push_back(nueva->first[0]);
	dummy->push_back(nueva->first[1]);
	dummy->push_back(nueva->first[4]);
	dummy->push_back(nueva->first[5]);
	nueva->third.push_back(dummy);
	nueva->fourth.push_back(new punto(0.0f,0.0f,0.0f));

	dummy=new std::list<punto *>;
	dummy->push_back(nueva->first[0]);
	dummy->push_back(nueva->first[1]);
	dummy->push_back(nueva->first[4]);
	dummy->push_back(nueva->first[5]);
	nueva->third.push_back(dummy);
	nueva->fourth.push_back(new punto(0.0f,0.0f,0.0f));

	dummy=new std::list<punto *>;
	dummy->push_back(nueva->first[0]);
	dummy->push_back(nueva->first[1]);
	dummy->push_back(nueva->first[2]);
	dummy->push_back(nueva->first[3]);
	nueva->third.push_back(dummy);
	nueva->fourth.push_back(new punto(0.0f,0.0f,0.0f));
	
	dummy=new std::list<punto *>;
	dummy->push_back(nueva->first[0]);
	dummy->push_back(nueva->first[1]);
	dummy->push_back(nueva->first[2]);
	dummy->push_back(nueva->first[3]);
	nueva->third.push_back(dummy);
	nueva->fourth.push_back(new punto(0.0f,0.0f,0.0f));

	//for(int i=0;i!=5;i++) unfla(nueva);
	
	
	return nueva;
}

void fisiquear(figura *o,float delta)
{
	int t=o->first.size();
	for(int i=0;i!=t;++i)
	{
		punto acumulador(0.0f,0.0f,0.0f);
		punto *p=o->first[i];
		std::list<punto *> *l=o->third[i];
		std::list<punto *>::iterator j;
		for(j = l->begin(); j != l->end(); ++j)
		{
			punto dummy((*p)-(**j));
			float modulo=dummy.module();
			modulo -=1.0f;
			modulo *=delta;
			dummy.normalize();
			dummy*=modulo;
			acumulador-=dummy;
		}
		(*p)+=acumulador;
	}	
}

/* The main drawing function. */
void DrawGLScene(figura *o)
{
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// Clear The Screen And The Depth Buffer
  glLoadIdentity();				// Reset The View

  static float rot=0.0f;
  static int cont=0;

   if (rot > 2* PI *50 &&cont<4){ 
	  unfla (o);
	  rot=rot-2*PI*50;
	  cont++;
	}
  

/*
  if(unfla1==true) { unfla(o); unfla1=false};
  if(unfla2==true) unfla(o);unfla2=false};
  if(unfla3==true) unfla(o);unfla3=false};
  if(unfla4==true) unfla(o);unfla4=false};
*/
  glTranslatef(0.0f,0.0f,-6.0f);		// Move Left 1.5 Units And Into The Screen 6.0

  bool color=false;
  glRotatef(rot,0.0f,1.0f,0.4f);
  glBegin(GL_TRIANGLES);
  fisiquear(o,0.05f);
  for(std::list<cara>::iterator i=o->second.begin();i!=o->second.end();++i)
  {
	  float coldist;
	  coldist=(punto(0.8f,0.8f,0.8f)-(*(i->a))).module()/2;
	  glColor3f(coldist*0.0f,0.0f,coldist*1.0f);
	  glVertex3f((*(i->a))[0],(*(i->a))[1],(*(i->a))[2]);
	 
	  coldist=(punto(0.8f,0.8f,0.8f)-(*(i->b))).module()/2;
	  glColor3f(coldist*0.5f,color ? 1.0f : coldist*0.2f,color ? coldist*0.6f : coldist*1.0f);
	  glVertex3f((*(i->b))[0],(*(i->b))[1],(*(i->b))[2]);
	  
	  coldist=(punto(0.8f,0.8f,0.8f)-(*(i->c))).module()/2;
	  glColor3f(coldist*0.8f,color ? 0.0f : 0.0f,color ? coldist*1.0f : coldist*0.8f);
	  glVertex3f((*(i->c))[0],(*(i->c))[1],(*(i->c))[2]);
  }
  glEnd();
  rot+=0.1f;
  SDL_GL_SwapBuffers();
}

int main(int argc, char *argv[]) 
{  
	SDL_Init(SDL_INIT_VIDEO);
	SDL_SetVideoMode(1024,1024,0,SDL_OPENGL);
	SDL_WM_SetCaption("Bola", NULL);
	
	InitGL(1024,1024);
	figura *bola=hacer_bola();

	initmusic();
	playmusic();
	bool done = false;
	while (!done)
	{
		DrawGLScene(bola);
		
		SDL_Event event;
		if(SDL_PollEvent(&event))
			switch(event.type)
			{
				case SDL_QUIT:
					done = true;
					break;
				case SDL_KEYDOWN:
					if(event.key.keysym.sym == SDLK_ESCAPE)	done = true;
					break;
				default:
					break;
			}
		//refreshevents();
	}


	closemusic();

  SDL_Quit();
  return 0;
}
